/* This file is part of Validators
 *
 *  It's a merge of the HTML- and the CSSValidator
 *
 *  Copyright (C) 2001 by  Richard Moore <rich@kde.org>
 *                         Andreas Schlapbach <schlpbch@iam.unibe.ch>
 *
 *  for information how to write your own plugin see:
 *    http://developer.kde.org/documentation/tutorials/dot/writing-plugins.html
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Library General Public
 *  License as published by the Free Software Foundation; either
 *  version 2 of the License, or (at your option) any later version.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Library General Public License for more details.
 *
 *  You should have received a copy of the GNU Library General Public License
 *  along with this library; see the file COPYING.LIB.  If not, write to
 *  the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
 *  Boston, MA 02110-1301, USA.
 **/


#include <kcomponentdata.h>
#include <kmessagebox.h>
#include <klocale.h>
#include <kaction.h>
#include <khtml_part.h>
#include <kpluginfactory.h>
#include <kdebug.h>
#include <kaboutdata.h>
#include <kactionmenu.h>
#include "plugin_validators.h"
#include "validatorsdialog.h"
#include <kicon.h>
#include <kmenu.h>
#include <kactioncollection.h>

K_PLUGIN_FACTORY(PluginValidatorsFactory, registerPlugin<PluginValidators>();)
K_EXPORT_PLUGIN(PluginValidatorsFactory(KAboutData("validatorsplugin", 0, ki18n("Validate Web Page") , "1.0" )))


PluginValidators::PluginValidators( QObject* parent,
                                    const QVariantList & )
  : Plugin( parent ), m_configDialog(0), m_part(0)
{
  setComponentData(PluginValidatorsFactory::componentData());

  m_menu = new KActionMenu ( KIcon("validators"),i18n( "&Validate Web Page" ),
			     actionCollection());
  actionCollection()->addAction("validateWebpage", m_menu);
  m_menu->setDelayed( false );

  m_menu->menu()->addAction(KIcon("htmlvalidator"), i18n( "Validate &HTML" ),
			       this, SLOT(slotValidateHTML()));

  m_menu->menu()->addAction(KIcon("cssvalidator"), i18n( "Validate &CSS" ),
			       this, SLOT(slotValidateCSS()));

  m_menu->menu()->addAction(i18n( "Validate &Links" ), this, SLOT(slotValidateLinks()));

  m_menu->setEnabled( false );

  if ( parent)
    {
      m_menu->menu()->addAction(KIcon("configure"), i18n( "C&onfigure Validator..." ),
				   this, SLOT(slotConfigure()));

      m_part = qobject_cast<KParts::ReadOnlyPart *>( parent );
      m_configDialog = new ValidatorsDialog( m_part->widget() );
      setURLs();

      connect( m_part, SIGNAL(started(KIO::Job*)), this,
             SLOT(slotStarted(KIO::Job*)) );
    }
}

PluginValidators::~PluginValidators()
{
  delete m_configDialog;
// Dont' delete the action. KActionCollection as parent does the job already
// and not deleting it at this point also ensures that in case we are not unplugged
// from the GUI yet and the ~KXMLGUIClient destructor will do so it won't hit a
// dead pointer. The kxmlgui factory keeps references to the actions, but it does not
// connect to their destroyed() signal, yet (need to find an elegant solution for that
// as it can easily increase the memory usage significantly) . That's why actions must
// persist as long as the plugin is plugged into the GUI.
//  delete m_menu;
}

void PluginValidators::setURLs()
{
  m_WWWValidatorUrl = KUrl(m_configDialog->getWWWValidatorUrl());
  m_CSSValidatorUrl = KUrl(m_configDialog->getCSSValidatorUrl());
  m_WWWValidatorUploadUrl = KUrl(m_configDialog->getWWWValidatorUploadUrl());
  m_CSSValidatorUploadUrl = KUrl(m_configDialog->getCSSValidatorUploadUrl());
  m_linkValidatorUrl = KUrl(m_configDialog->getLinkValidatorUrl());
}

void PluginValidators::slotStarted( KIO::Job* )
{
  // The w3c validator can only access http URLs, and upload local files.
  // No https, probably no webdav either.
  m_menu->setEnabled( m_part->url().isLocalFile()
                      || m_part->url().protocol().toLower() == "http" );
}

void PluginValidators::slotValidateHTML()
{
  validateURL(m_WWWValidatorUrl, m_WWWValidatorUploadUrl);
}

void PluginValidators::slotValidateCSS()
{
  validateURL(m_CSSValidatorUrl, m_CSSValidatorUploadUrl);
}

void PluginValidators::slotValidateLinks()
{
  validateURL(m_linkValidatorUrl, KUrl());
}

void PluginValidators::slotConfigure()
{
  m_configDialog->show();
  setURLs();
}

void PluginValidators::validateURL(const KUrl &url, const KUrl &uploadUrl)
{
  // The parent is assumed to be a KHTMLPart
  if ( !parent()->inherits("KHTMLPart") &&!parent()->inherits("WebKitPart")  )
    {
      QString title = i18n( "Cannot Validate Source" );
      QString text = i18n( "You cannot validate anything except web pages with "
			   "this plugin." );

      KMessageBox::sorry( 0, text, title );
      return;
    }

  KUrl validatorUrl(url);

  // Get URL
  KUrl partUrl = m_part->url();
  if ( !partUrl.isValid() ) // Just in case ;)
    {
      QString title = i18n( "Malformed URL" );
      QString text = i18n( "The URL you entered is not valid, please "
			   "correct it and try again." );
      KMessageBox::sorry( 0, text, title );
      return;
    }

  if (partUrl.isLocalFile())
    {
      if ( validatorUrl.isEmpty() ) {
        QString title = i18n( "Upload Not Possible" );
        QString text = i18n( "Validating links is not possible for local "
                             "files." );
        KMessageBox::sorry( 0, text, title );
        return;
      }
      validatorUrl = uploadUrl;
    }
  else
    {
      if (partUrl.hasPass())
        {
          KMessageBox::sorry(
          	m_part->widget(),
          	i18n("<qt>The selected URL cannot be verified because it contains "
          	     "a password. Sending this URL to <b>%1</b> would put the security "
          	     "of <b>%2</b> at risk.</qt>",
          	      validatorUrl.host(), partUrl.host()));
          return;
        }
      // Set entered URL as a parameter
      QString q = partUrl.url();
      q = KUrl::encode_string( q );
      QString p = "uri=";
      p += q;
      validatorUrl.setQuery( p );
    }
  kDebug(90120) << "final URL: " << validatorUrl.url();
  KParts::BrowserExtension * ext = KParts::BrowserExtension::childObject(m_part);
  emit ext->openUrlRequest( validatorUrl );
}

#include <plugin_validators.moc>
